@@ -1,2 +1,2 @@
 from .plot import mis, gen_cam, plot_act_cam, plot_act_heatmap, plot_pred_cam, plot_pred_heatmap, plot_cycle, true_list
-from .graphs import acc_loss, testvtrain, class_acc
\ No newline at end of file
+from .graphs import acc_loss, testvtrain, class_acc, classes
\ No newline at end of file
